@@ -9,8 +9,8 @@
 use std::{borrow::Cow, ffi::OsStr};
 
 use crate::native_int_str::{
-    from_native_int_representation, get_char_from_native_int, get_single_native_int_value,
-    NativeCharInt, NativeIntStr,
+    NativeCharInt, NativeIntStr, from_native_int_representation, get_char_from_native_int,
+    get_single_native_int_value,
 };
 
 #[derive(Clone, Debug, Eq, PartialEq)]
@@ -114,10 +114,9 @@ impl<'a> StringParser<'a> {
     }
 
     pub fn peek_chunk(&self) -> Option<Chunk<'a>> {
-        return self
-            .get_chunk_with_length_at(self.pointer)
+        self.get_chunk_with_length_at(self.pointer)
             .ok()
-            .map(|(chunk, _)| chunk);
+            .map(|(chunk, _)| chunk)
     }
 
     pub fn consume_chunk(&mut self) -> Result<Chunk<'a>, Error> {
