@@ -5,18 +5,23 @@
 
 // spell-checker:ignore (ToDO) signalname pids killpg
 
-use clap::{crate_version, Arg, ArgAction, Command};
+use clap::{Arg, ArgAction, Command};
 use nix::sys::signal::{self, Signal};
 use nix::unistd::Pid;
 use std::io::Error;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError};
-use uucore::signals::{signal_by_name_or_value, signal_name_by_value, ALL_SIGNALS};
+use uucore::signals::{ALL_SIGNALS, signal_by_name_or_value, signal_name_by_value};
 use uucore::{format_usage, help_about, help_usage, show};
 
 static ABOUT: &str = help_about!("kill.md");
 const USAGE: &str = help_usage!("kill.md");
 
+// When the -l option is selected, the program displays the type of signal related to a certain
+// value or string. In case of a value, the program should control the lower 8 bits, but there is
+// a particular case in which if the value is in range [128, 159], it is translated to a signal
+const OFFSET: usize = 128;
+
 pub mod options {
     pub static PIDS_OR_SIGNALS: &str = "pids_or_signals";
     pub static LIST: &str = "list";
@@ -69,7 +74,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             } else {
                 let sig = (sig as i32)
                     .try_into()
-                    .map_err(|e| std::io::Error::from_raw_os_error(e as i32))?;
+                    .map_err(|e| Error::from_raw_os_error(e as i32))?;
                 Some(sig)
             };
 
@@ -98,7 +103,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .version(crate_version!())
+        .version(uucore::crate_version!())
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .infer_long_args(true)
@@ -141,6 +146,10 @@ fn handle_obsolete(args: &mut Vec<String>) -> Option<usize> {
         // Old signal can only be in the first argument position
         let slice = args[1].as_str();
         if let Some(signal) = slice.strip_prefix('-') {
+            // With '-', a signal name must start with an uppercase char
+            if signal.chars().next().is_some_and(|c| c.is_lowercase()) {
+                return None;
+            }
             // Check if it is a valid signal
             let opt_signal = signal_by_name_or_value(signal);
             if opt_signal.is_some() {
@@ -154,37 +163,42 @@ fn handle_obsolete(args: &mut Vec<String>) -> Option<usize> {
 }
 
 fn table() {
-    // GNU kill doesn't list the EXIT signal with --table, so we ignore it, too
-    for (idx, signal) in ALL_SIGNALS
-        .iter()
-        .enumerate()
-        .filter(|(_, s)| **s != "EXIT")
-    {
+    for (idx, signal) in ALL_SIGNALS.iter().enumerate() {
         println!("{idx: >#2} {signal}");
     }
 }
 
 fn print_signal(signal_name_or_value: &str) -> UResult<()> {
+    // Closure used to track the last 8 bits of the signal value
+    // when the -l option is passed only the lower 8 bits are important
+    // or the value is in range [128, 159]
+    // Example: kill -l 143 => TERM because 143 = 15 + 128
+    // Example: kill -l 2304 => EXIT
+    let lower_8_bits = |x: usize| x & 0xff;
+    let option_num_parse = signal_name_or_value.parse::<usize>().ok();
+
     for (value, &signal) in ALL_SIGNALS.iter().enumerate() {
         if signal.eq_ignore_ascii_case(signal_name_or_value)
             || format!("SIG{signal}").eq_ignore_ascii_case(signal_name_or_value)
         {
             println!("{value}");
             return Ok(());
-        } else if signal_name_or_value == value.to_string() {
+        } else if signal_name_or_value == value.to_string()
+            || option_num_parse.is_some_and(|signal_value| lower_8_bits(signal_value) == value)
+            || option_num_parse.is_some_and(|signal_value| signal_value == value + OFFSET)
+        {
             println!("{signal}");
             return Ok(());
         }
     }
     Err(USimpleError::new(
         1,
-        format!("unknown signal name {}", signal_name_or_value.quote()),
+        format!("{}: invalid signal", signal_name_or_value.quote()),
     ))
 }
 
 fn print_signals() {
-    // GNU kill doesn't list the EXIT signal with --list, so we ignore it, too
-    for signal in ALL_SIGNALS.iter().filter(|x| **x != "EXIT") {
+    for signal in ALL_SIGNALS {
         println!("{signal}");
     }
 }
@@ -207,7 +221,7 @@ fn parse_signal_value(signal_name: &str) -> UResult<usize> {
         Some(x) => Ok(x),
         None => Err(USimpleError::new(
             1,
-            format!("unknown signal name {}", signal_name.quote()),
+            format!("{}: invalid signal", signal_name.quote()),
         )),
     }
 }
@@ -216,7 +230,7 @@ fn parse_pids(pids: &[String]) -> UResult<Vec<i32>> {
     pids.iter()
         .map(|x| {
             x.parse::<i32>().map_err(|e| {
-                USimpleError::new(1, format!("failed to parse argument {}: {}", x.quote(), e))
+                USimpleError::new(1, format!("failed to parse argument {}: {e}", x.quote()))
             })
         })
         .collect()
@@ -225,8 +239,10 @@ fn parse_pids(pids: &[String]) -> UResult<Vec<i32>> {
 fn kill(sig: Option<Signal>, pids: &[i32]) {
     for &pid in pids {
         if let Err(e) = signal::kill(Pid::from_raw(pid), sig) {
-            show!(Error::from_raw_os_error(e as i32)
-                .map_err_context(|| format!("sending signal to {pid} failed")));
+            show!(
+                Error::from_raw_os_error(e as i32)
+                    .map_err_context(|| format!("sending signal to {pid} failed"))
+            );
         }
     }
 }
