@@ -5,24 +5,23 @@
 
 // spell-checker:ignore (ToDOs) corasick memchr Roff trunc oset iset CHARCLASS
 
-use clap::{crate_version, Arg, ArgAction, Command};
-use regex::Regex;
 use std::cmp;
 use std::collections::{BTreeSet, HashMap, HashSet};
-use std::error::Error;
-use std::fmt::{Display, Formatter, Write as FmtWrite};
+use std::fmt::Write as FmtWrite;
 use std::fs::File;
-use std::io::{stdin, stdout, BufRead, BufReader, BufWriter, Read, Write};
+use std::io::{BufRead, BufReader, BufWriter, Read, Write, stdin, stdout};
 use std::num::ParseIntError;
+
+use clap::{Arg, ArgAction, Command};
+use regex::Regex;
+use thiserror::Error;
 use uucore::display::Quotable;
-use uucore::error::{FromIo, UError, UResult};
+use uucore::error::{FromIo, UError, UResult, UUsageError};
 use uucore::{format_usage, help_about, help_usage};
 
 const USAGE: &str = help_usage!("ptx.md");
 const ABOUT: &str = help_about!("ptx.md");
 
-const REGEX_CHARCLASS: &str = "^-]\\";
-
 #[derive(Debug)]
 enum OutFormat {
     Dumb,
@@ -71,8 +70,12 @@ fn read_word_filter_file(
         .get_one::<String>(option)
         .expect("parsing options failed!")
         .to_string();
-    let file = File::open(filename)?;
-    let reader = BufReader::new(file);
+    let reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
+        Box::new(stdin())
+    } else {
+        let file = File::open(filename)?;
+        Box::new(file)
+    });
     let mut words: HashSet<String> = HashSet::new();
     for word in reader.lines() {
         words.insert(word?);
@@ -88,7 +91,12 @@ fn read_char_filter_file(
     let filename = matches
         .get_one::<String>(option)
         .expect("parsing options failed!");
-    let mut reader = File::open(filename)?;
+    let mut reader: Box<dyn Read> = if filename == "-" {
+        Box::new(stdin())
+    } else {
+        let file = File::open(filename)?;
+        Box::new(file)
+    };
     let mut buffer = String::new();
     reader.read_to_string(&mut buffer)?;
     Ok(buffer.chars().collect())
@@ -129,7 +137,7 @@ impl WordFilter {
                 HashSet::new() // really only chars found in file
             } else {
                 // GNU off means at least these are considered
-                [' ', '\t', '\n'].iter().cloned().collect()
+                [' ', '\t', '\n'].iter().copied().collect()
             };
             hs.extend(chars);
             Some(hs)
@@ -155,18 +163,10 @@ impl WordFilter {
         let reg = match arg_reg {
             Some(arg_reg) => arg_reg,
             None => {
-                if break_set.is_some() {
+                if let Some(break_set) = break_set {
                     format!(
                         "[^{}]+",
-                        break_set
-                            .unwrap()
-                            .into_iter()
-                            .map(|c| if REGEX_CHARCLASS.contains(c) {
-                                format!("\\{c}")
-                            } else {
-                                c.to_string()
-                            })
-                            .collect::<String>()
+                        regex::escape(&break_set.into_iter().collect::<String>())
                     )
                 } else if config.gnu_ext {
                     "\\w+".to_owned()
@@ -195,28 +195,18 @@ struct WordRef {
     filename: String,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Error)]
 enum PtxError {
+    #[error("There is no dumb format with GNU extensions disabled")]
     DumbFormat,
+    #[error("{0} not implemented yet")]
     NotImplemented(&'static str),
+    #[error("{0}")]
     ParseError(ParseIntError),
 }
 
-impl Error for PtxError {}
 impl UError for PtxError {}
 
-impl Display for PtxError {
-    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
-        match self {
-            Self::DumbFormat => {
-                write!(f, "There is no dumb format with GNU extensions disabled")
-            }
-            Self::NotImplemented(s) => write!(f, "{s} not implemented yet"),
-            Self::ParseError(e) => e.fmt(f),
-        }
-    }
-}
-
 fn get_config(matches: &clap::ArgMatches) -> UResult<Config> {
     let mut config = Config::default();
     let err_msg = "parsing options failed";
@@ -260,10 +250,17 @@ fn get_config(matches: &clap::ArgMatches) -> UResult<Config> {
             .parse()
             .map_err(PtxError::ParseError)?;
     }
-    if matches.get_flag(options::FORMAT_ROFF) {
+    if let Some(format) = matches.get_one::<String>(options::FORMAT) {
+        config.format = match format.as_str() {
+            "roff" => OutFormat::Roff,
+            "tex" => OutFormat::Tex,
+            _ => unreachable!("should be caught by clap"),
+        };
+    }
+    if matches.get_flag(options::format::ROFF) {
         config.format = OutFormat::Roff;
     }
-    if matches.get_flag(options::FORMAT_TEX) {
+    if matches.get_flag(options::format::TEX) {
         config.format = OutFormat::Tex;
     }
     Ok(config)
@@ -277,20 +274,10 @@ struct FileContent {
 
 type FileMap = HashMap<String, FileContent>;
 
-fn read_input(input_files: &[String], config: &Config) -> std::io::Result<FileMap> {
+fn read_input(input_files: &[String]) -> std::io::Result<FileMap> {
     let mut file_map: FileMap = HashMap::new();
-    let mut files = Vec::new();
-    if input_files.is_empty() {
-        files.push("-");
-    } else if config.gnu_ext {
-        for file in input_files {
-            files.push(file);
-        }
-    } else {
-        files.push(&input_files[0]);
-    }
     let mut offset: usize = 0;
-    for filename in files {
+    for filename in input_files {
         let reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
             Box::new(stdin())
         } else {
@@ -337,14 +324,14 @@ fn create_word_set(config: &Config, filter: &WordFilter, file_map: &FileMap) ->
                     continue;
                 }
                 let mut word = line[beg..end].to_owned();
-                if filter.only_specified && !(filter.only_set.contains(&word)) {
+                if filter.only_specified && !filter.only_set.contains(&word) {
                     continue;
                 }
                 if filter.ignore_specified && filter.ignore_set.contains(&word) {
                     continue;
                 }
                 if config.ignore_case {
-                    word = word.to_lowercase();
+                    word = word.to_uppercase();
                 }
                 word_set.insert(WordRef {
                     word,
@@ -533,9 +520,9 @@ fn get_output_chunks(
 
     // put left context truncation string if needed
     if before_beg != 0 && head_beg == head_end {
-        before = format!("{}{}", config.trunc_str, before);
+        before = format!("{}{before}", config.trunc_str);
     } else if before_beg != 0 && head_beg != 0 {
-        head = format!("{}{}", config.trunc_str, head);
+        head = format!("{}{head}", config.trunc_str);
     }
 
     (tail, before, after, head)
@@ -565,26 +552,14 @@ fn format_tex_line(
 ) -> String {
     let mut output = String::new();
     write!(output, "\\{} ", config.macro_name).unwrap();
-    let all_before = if config.input_ref {
-        let before = &line[0..word_ref.position];
-        let before_start_trim_offset =
-            word_ref.position - before.trim_start_matches(reference).trim_start().len();
-        let before_end_index = before.len();
-        &chars_line[before_start_trim_offset..cmp::max(before_end_index, before_start_trim_offset)]
-    } else {
-        let before_chars_trim_idx = (0, word_ref.position);
-        &chars_line[before_chars_trim_idx.0..before_chars_trim_idx.1]
-    };
-    let keyword = &line[word_ref.position..word_ref.position_end];
-    let after_chars_trim_idx = (word_ref.position_end, chars_line.len());
-    let all_after = &chars_line[after_chars_trim_idx.0..after_chars_trim_idx.1];
-    let (tail, before, after, head) = get_output_chunks(all_before, keyword, all_after, config);
+    let (tail, before, keyword, after, head) =
+        prepare_line_chunks(config, word_ref, line, chars_line, reference);
     write!(
         output,
         "{{{0}}}{{{1}}}{{{2}}}{{{3}}}{{{4}}}",
         format_tex_field(&tail),
         format_tex_field(&before),
-        format_tex_field(keyword),
+        format_tex_field(&keyword),
         format_tex_field(&after),
         format_tex_field(&head),
     )
@@ -608,26 +583,14 @@ fn format_roff_line(
 ) -> String {
     let mut output = String::new();
     write!(output, ".{}", config.macro_name).unwrap();
-    let all_before = if config.input_ref {
-        let before = &line[0..word_ref.position];
-        let before_start_trim_offset =
-            word_ref.position - before.trim_start_matches(reference).trim_start().len();
-        let before_end_index = before.len();
-        &chars_line[before_start_trim_offset..cmp::max(before_end_index, before_start_trim_offset)]
-    } else {
-        let before_chars_trim_idx = (0, word_ref.position);
-        &chars_line[before_chars_trim_idx.0..before_chars_trim_idx.1]
-    };
-    let keyword = &line[word_ref.position..word_ref.position_end];
-    let after_chars_trim_idx = (word_ref.position_end, chars_line.len());
-    let all_after = &chars_line[after_chars_trim_idx.0..after_chars_trim_idx.1];
-    let (tail, before, after, head) = get_output_chunks(all_before, keyword, all_after, config);
+    let (tail, before, keyword, after, head) =
+        prepare_line_chunks(config, word_ref, line, chars_line, reference);
     write!(
         output,
         " \"{}\" \"{}\" \"{}{}\" \"{}\"",
         format_roff_field(&tail),
         format_roff_field(&before),
-        format_roff_field(keyword),
+        format_roff_field(&keyword),
         format_roff_field(&after),
         format_roff_field(&head)
     )
@@ -638,6 +601,46 @@ fn format_roff_line(
     output
 }
 
+/// Extract and prepare text chunks for formatting in both TeX and roff output
+fn prepare_line_chunks(
+    config: &Config,
+    word_ref: &WordRef,
+    line: &str,
+    chars_line: &[char],
+    reference: &str,
+) -> (String, String, String, String, String) {
+    // Convert byte positions to character positions
+    let ref_char_position = line[..word_ref.position].chars().count();
+    let char_position_end = ref_char_position
+        + line[word_ref.position..word_ref.position_end]
+            .chars()
+            .count();
+
+    // Extract the text before the keyword
+    let all_before = if config.input_ref {
+        let before = &line[..word_ref.position];
+        let before_char_count = before.chars().count();
+        let trimmed_char_count = before
+            .trim_start_matches(reference)
+            .trim_start()
+            .chars()
+            .count();
+        let trim_offset = before_char_count - trimmed_char_count;
+        &chars_line[trim_offset..before_char_count]
+    } else {
+        &chars_line[..ref_char_position]
+    };
+
+    // Extract the keyword and text after it
+    let keyword = line[word_ref.position..word_ref.position_end].to_string();
+    let all_after = &chars_line[char_position_end..];
+
+    // Get formatted output chunks
+    let (tail, before, after, head) = get_output_chunks(all_before, &keyword, all_after, config);
+
+    (tail, before, keyword, after, head)
+}
+
 fn write_traditional_output(
     config: &Config,
     file_map: &FileMap,
@@ -647,7 +650,8 @@ fn write_traditional_output(
     let mut writer: BufWriter<Box<dyn Write>> = BufWriter::new(if output_filename == "-" {
         Box::new(stdout())
     } else {
-        let file = File::create(output_filename).map_err_context(String::new)?;
+        let file = File::create(output_filename)
+            .map_err_context(|| output_filename.maybe_quote().to_string())?;
         Box::new(file)
     });
 
@@ -687,21 +691,28 @@ fn write_traditional_output(
                 return Err(PtxError::DumbFormat.into());
             }
         };
-        writeln!(writer, "{output_line}").map_err_context(String::new)?;
+        writeln!(writer, "{output_line}").map_err_context(|| "write failed".into())?;
     }
+
+    writer.flush().map_err_context(|| "write failed".into())?;
+
     Ok(())
 }
 
 mod options {
+    pub mod format {
+        pub static ROFF: &str = "roff";
+        pub static TEX: &str = "tex";
+    }
+
     pub static FILE: &str = "file";
     pub static AUTO_REFERENCE: &str = "auto-reference";
     pub static TRADITIONAL: &str = "traditional";
     pub static FLAG_TRUNCATION: &str = "flag-truncation";
     pub static MACRO_NAME: &str = "macro-name";
-    pub static FORMAT_ROFF: &str = "format=roff";
+    pub static FORMAT: &str = "format";
     pub static RIGHT_SIDE_REFS: &str = "right-side-refs";
     pub static SENTENCE_REGEXP: &str = "sentence-regexp";
-    pub static FORMAT_TEX: &str = "format=tex";
     pub static WORD_REGEXP: &str = "word-regexp";
     pub static BREAK_FILE: &str = "break-file";
     pub static IGNORE_CASE: &str = "ignore-case";
@@ -715,28 +726,47 @@ mod options {
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let matches = uu_app().try_get_matches_from(args)?;
-
-    let mut input_files: Vec<String> = match &matches.get_many::<String>(options::FILE) {
-        Some(v) => v.clone().cloned().collect(),
-        None => vec!["-".to_string()],
-    };
-
     let config = get_config(&matches)?;
-    let word_filter = WordFilter::new(&matches, &config)?;
-    let file_map = read_input(&input_files, &config).map_err_context(String::new)?;
-    let word_set = create_word_set(&config, &word_filter, &file_map);
-    let output_file = if !config.gnu_ext && input_files.len() == 2 {
-        input_files.pop().unwrap()
+
+    let input_files;
+    let output_file;
+
+    let mut files = matches
+        .get_many::<String>(options::FILE)
+        .into_iter()
+        .flatten()
+        .cloned();
+
+    if !config.gnu_ext {
+        input_files = vec![files.next().unwrap_or("-".to_string())];
+        output_file = files.next().unwrap_or("-".to_string());
+        if let Some(file) = files.next() {
+            return Err(UUsageError::new(
+                1,
+                format!("extra operand {}", file.quote()),
+            ));
+        }
     } else {
-        "-".to_string()
-    };
+        input_files = {
+            let mut files = files.collect::<Vec<_>>();
+            if files.is_empty() {
+                files.push("-".to_string());
+            }
+            files
+        };
+        output_file = "-".to_string();
+    }
+
+    let word_filter = WordFilter::new(&matches, &config)?;
+    let file_map = read_input(&input_files).map_err_context(String::new)?;
+    let word_set = create_word_set(&config, &word_filter, &file_map);
     write_traditional_output(&config, &file_map, &word_set, &output_file)
 }
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .about(ABOUT)
-        .version(crate_version!())
+        .version(uucore::crate_version!())
         .override_usage(format_usage(USAGE))
         .infer_long_args(true)
         .arg(
@@ -774,10 +804,24 @@ pub fn uu_app() -> Command {
                 .value_name("STRING"),
         )
         .arg(
-            Arg::new(options::FORMAT_ROFF)
+            Arg::new(options::FORMAT)
+                .long(options::FORMAT)
+                .hide(true)
+                .value_parser(["roff", "tex"])
+                .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX]),
+        )
+        .arg(
+            Arg::new(options::format::ROFF)
                 .short('O')
-                .long(options::FORMAT_ROFF)
                 .help("generate output as roff directives")
+                .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX])
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::format::TEX)
+                .short('T')
+                .help("generate output as TeX directives")
+                .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX])
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -794,13 +838,6 @@ pub fn uu_app() -> Command {
                 .help("for end of lines or end of sentences")
                 .value_name("REGEXP"),
         )
-        .arg(
-            Arg::new(options::FORMAT_TEX)
-                .short('T')
-                .long(options::FORMAT_TEX)
-                .help("generate output as TeX directives")
-                .action(ArgAction::SetTrue),
-        )
         .arg(
             Arg::new(options::WORD_REGEXP)
                 .short('W')
