@@ -5,10 +5,10 @@
 
 // spell-checker:ignore (words) wipesync prefill
 
-use clap::{crate_version, Arg, ArgAction, Command};
+use clap::{Arg, ArgAction, Command};
 #[cfg(unix)]
 use libc::S_IWUSR;
-use rand::{rngs::StdRng, seq::SliceRandom, Rng, SeedableRng};
+use rand::{Rng, SeedableRng, rngs::StdRng, seq::SliceRandom};
 use std::fs::{self, File, OpenOptions};
 use std::io::{self, Seek, Write};
 #[cfg(unix)]
@@ -16,8 +16,8 @@ use std::os::unix::prelude::PermissionsExt;
 use std::path::{Path, PathBuf};
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError, UUsageError};
-use uucore::parse_size::parse_size_u64;
-use uucore::shortcut_value_parser::ShortcutValueParser;
+use uucore::parser::parse_size::parse_size_u64;
+use uucore::parser::shortcut_value_parser::ShortcutValueParser;
 use uucore::{format_usage, help_about, help_section, help_usage, show_error, show_if_err};
 
 const ABOUT: &str = help_about!("shred.md");
@@ -49,10 +49,15 @@ const NAME_CHARSET: &[u8] = b"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMN
 const PATTERN_LENGTH: usize = 3;
 const PATTERN_BUFFER_SIZE: usize = BLOCK_SIZE + PATTERN_LENGTH - 1;
 
+/// Optimal block size for the filesystem. This constant is used for data size alignment, similar
+/// to the behavior of GNU shred. Usually, optimal block size is a 4K block (2^12), which is why
+/// it's defined as a constant. However, it's possible to get the actual size at runtime using, for
+/// example, `std::os::unix::fs::MetadataExt::blksize()`.
+const OPTIMAL_IO_BLOCK_SIZE: usize = 1 << 12;
+
 /// Patterns that appear in order for the passes
 ///
-/// They are all extended to 3 bytes for consistency, even though some could be
-/// expressed as single bytes.
+/// A single-byte pattern is equivalent to a multi-byte pattern of that byte three times.
 const PATTERNS: [Pattern; 22] = [
     Pattern::Single(b'\x00'),
     Pattern::Single(b'\xFF'),
@@ -176,7 +181,7 @@ impl BytesWriter {
     fn from_pass_type(pass: &PassType) -> Self {
         match pass {
             PassType::Random => Self::Random {
-                rng: StdRng::from_entropy(),
+                rng: StdRng::from_os_rng(),
                 buffer: [0; BLOCK_SIZE],
             },
             PassType::Pattern(pattern) => {
@@ -229,7 +234,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 return Err(USimpleError::new(
                     1,
                     format!("invalid number of passes: {}", s.quote()),
-                ))
+                ));
             }
         },
         None => unreachable!(),
@@ -279,7 +284,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .version(crate_version!())
+        .version(uucore::crate_version!())
         .about(ABOUT)
         .after_help(AFTER_HELP)
         .override_usage(format_usage(USAGE))
@@ -376,8 +381,8 @@ fn get_size(size_str_opt: Option<String>) -> Option<u64> {
 fn pass_name(pass_type: &PassType) -> String {
     match pass_type {
         PassType::Random => String::from("random"),
-        PassType::Pattern(Pattern::Single(byte)) => format!("{byte:x}{byte:x}{byte:x}"),
-        PassType::Pattern(Pattern::Multi([a, b, c])) => format!("{a:x}{b:x}{c:x}"),
+        PassType::Pattern(Pattern::Single(byte)) => format!("{byte:02x}{byte:02x}{byte:02x}"),
+        PassType::Pattern(Pattern::Multi([a, b, c])) => format!("{a:02x}{b:02x}{c:02x}"),
     }
 }
 
@@ -440,9 +445,13 @@ fn wipe_file(
                 pass_sequence.push(PassType::Random);
             }
         } else {
-            // First fill it with Patterns, shuffle it, then evenly distribute Random
-            let n_full_arrays = n_passes / PATTERNS.len(); // How many times can we go through all the patterns?
-            let remainder = n_passes % PATTERNS.len(); // How many do we get through on our last time through?
+            // Add initial random to avoid O(n) operation later
+            pass_sequence.push(PassType::Random);
+            let n_random = (n_passes / 10).max(3); // Minimum 3 random passes; ratio of 10 after
+            let n_fixed = n_passes - n_random;
+            // Fill it with Patterns and all but the first and last random, then shuffle it
+            let n_full_arrays = n_fixed / PATTERNS.len(); // How many times can we go through all the patterns?
+            let remainder = n_fixed % PATTERNS.len(); // How many do we get through on our last time through, excluding randoms?
 
             for _ in 0..n_full_arrays {
                 for p in PATTERNS {
@@ -452,14 +461,14 @@ fn wipe_file(
             for pattern in PATTERNS.into_iter().take(remainder) {
                 pass_sequence.push(PassType::Pattern(pattern));
             }
-            let mut rng = rand::thread_rng();
-            pass_sequence.shuffle(&mut rng); // randomize the order of application
-
-            let n_random = 3 + n_passes / 10; // Minimum 3 random passes; ratio of 10 after
-                                              // Evenly space random passes; ensures one at the beginning and end
-            for i in 0..n_random {
-                pass_sequence[i * (n_passes - 1) / (n_random - 1)] = PassType::Random;
+            // add random passes except one each at the beginning and end
+            for _ in 0..n_random - 2 {
+                pass_sequence.push(PassType::Random);
             }
+
+            let mut rng = rand::rng();
+            pass_sequence[1..].shuffle(&mut rng); // randomize the order of application
+            pass_sequence.push(PassType::Random); // add the last random pass
         }
 
         // --zero specifies whether we want one final pass of 0x00 on our file
@@ -484,17 +493,17 @@ fn wipe_file(
         if verbose {
             let pass_name = pass_name(&pass_type);
             show_error!(
-                "{}: pass {:2}/{} ({})...",
+                "{}: pass {}/{total_passes} ({pass_name})...",
                 path.maybe_quote(),
                 i + 1,
-                total_passes,
-                pass_name
             );
         }
         // size is an optional argument for exactly how many bytes we want to shred
         // Ignore failed writes; just keep trying
-        show_if_err!(do_pass(&mut file, &pass_type, exact, size)
-            .map_err_context(|| format!("{}: File write pass failed", path.maybe_quote())));
+        show_if_err!(
+            do_pass(&mut file, &pass_type, exact, size)
+                .map_err_context(|| format!("{}: File write pass failed", path.maybe_quote()))
+        );
     }
 
     if remove_method != RemoveMethod::None {
@@ -504,6 +513,23 @@ fn wipe_file(
     Ok(())
 }
 
+fn split_on_blocks(file_size: u64, exact: bool) -> (u64, u64) {
+    // OPTIMAL_IO_BLOCK_SIZE must not exceed BLOCK_SIZE. Violating this may cause overflows due
+    // to alignment or performance issues.This kind of misconfiguration is
+    // highly unlikely but would indicate a serious error.
+    const _: () = assert!(OPTIMAL_IO_BLOCK_SIZE <= BLOCK_SIZE);
+
+    let file_size = if exact {
+        file_size
+    } else {
+        // The main idea here is to align the file size to the OPTIMAL_IO_BLOCK_SIZE, and then
+        // split it into BLOCK_SIZE + remaining bytes. Since the input data is already aligned to N
+        // * OPTIMAL_IO_BLOCK_SIZE, the output file size will also be aligned and correct.
+        file_size.div_ceil(OPTIMAL_IO_BLOCK_SIZE as u64) * OPTIMAL_IO_BLOCK_SIZE as u64
+    };
+    (file_size / BLOCK_SIZE as u64, file_size % BLOCK_SIZE as u64)
+}
+
 fn do_pass(
     file: &mut File,
     pass_type: &PassType,
@@ -514,21 +540,17 @@ fn do_pass(
     file.rewind()?;
 
     let mut writer = BytesWriter::from_pass_type(pass_type);
+    let (number_of_blocks, bytes_left) = split_on_blocks(file_size, exact);
 
     // We start by writing BLOCK_SIZE times as many time as possible.
-    for _ in 0..(file_size / BLOCK_SIZE as u64) {
+    for _ in 0..number_of_blocks {
         let block = writer.bytes_for_pass(BLOCK_SIZE);
         file.write_all(block)?;
     }
 
-    // Now we might have some bytes left, so we write either that
-    // many bytes if exact is true, or BLOCK_SIZE bytes if not.
-    let bytes_left = (file_size % BLOCK_SIZE as u64) as usize;
-    if bytes_left > 0 {
-        let size = if exact { bytes_left } else { BLOCK_SIZE };
-        let block = writer.bytes_for_pass(size);
-        file.write_all(block)?;
-    }
+    // Then we write remaining data which is smaller than the BLOCK_SIZE
+    let block = writer.bytes_for_pass(bytes_left as usize);
+    file.write_all(block)?;
 
     file.sync_data()?;
 
@@ -576,10 +598,9 @@ fn wipe_name(orig_path: &Path, verbose: bool, remove_method: RemoveMethod) -> Op
                 }
                 Err(e) => {
                     show_error!(
-                        "{}: Couldn't rename to {}: {}",
+                        "{}: Couldn't rename to {}: {e}",
                         last_path.maybe_quote(),
                         new_path.quote(),
-                        e
                     );
                     // TODO: replace with our error management
                     std::process::exit(1);
@@ -617,3 +638,40 @@ fn do_remove(
 
     Ok(())
 }
+
+#[cfg(test)]
+mod tests {
+
+    use crate::{BLOCK_SIZE, OPTIMAL_IO_BLOCK_SIZE, split_on_blocks};
+
+    #[test]
+    fn test_align_non_exact_control_values() {
+        // Note: This test only makes sense for the default values of BLOCK_SIZE and
+        // OPTIMAL_IO_BLOCK_SIZE.
+        assert_eq!(split_on_blocks(1, false), (0, 4096));
+        assert_eq!(split_on_blocks(4095, false), (0, 4096));
+        assert_eq!(split_on_blocks(4096, false), (0, 4096));
+        assert_eq!(split_on_blocks(4097, false), (0, 8192));
+        assert_eq!(split_on_blocks(65535, false), (1, 0));
+        assert_eq!(split_on_blocks(65536, false), (1, 0));
+        assert_eq!(split_on_blocks(65537, false), (1, 4096));
+    }
+
+    #[test]
+    fn test_align_non_exact_cycle() {
+        for size in 1..BLOCK_SIZE as u64 * 2 {
+            let (number_of_blocks, bytes_left) = split_on_blocks(size, false);
+            let test_size = number_of_blocks * BLOCK_SIZE as u64 + bytes_left;
+            assert_eq!(test_size % OPTIMAL_IO_BLOCK_SIZE as u64, 0);
+        }
+    }
+
+    #[test]
+    fn test_align_exact_cycle() {
+        for size in 1..BLOCK_SIZE as u64 * 2 {
+            let (number_of_blocks, bytes_left) = split_on_blocks(size, true);
+            let test_size = number_of_blocks * BLOCK_SIZE as u64 + bytes_left;
+            assert_eq!(test_size, size);
+        }
+    }
+}
