@@ -2,7 +2,9 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-use crate::common::util::TestScenario;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
 
 #[test]
 fn test_encode() {
@@ -72,7 +74,7 @@ fn test_base64_encode_file() {
 
 #[test]
 fn test_decode() {
-    for decode_param in ["-d", "--decode", "--dec"] {
+    for decode_param in ["-d", "--decode", "--dec", "-D"] {
         let input = "aGVsbG8sIHdvcmxkIQ=="; // spell-checker:disable-line
         new_ucmd!()
             .arg(decode_param)
@@ -232,7 +234,7 @@ fn test_manpage() {
 
     let test_scenario = TestScenario::new("");
 
-    let child = Command::new(test_scenario.bin_path)
+    let child = Command::new(&test_scenario.bin_path)
         .arg("manpage")
         .arg("base64")
         .stdin(Stdio::piped())
