@@ -2,14 +2,17 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-use crate::common::util::TestScenario;
+
+use chrono::{DateTime, Datelike, Duration, NaiveTime, Utc}; // spell-checker:disable-line
 use regex::Regex;
 #[cfg(all(unix, not(target_os = "macos")))]
 use uucore::process::geteuid;
+use uutests::util::TestScenario;
+use uutests::{at_and_ucmd, new_ucmd, util_name};
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -163,6 +166,14 @@ fn test_date_format_y() {
     scene.ucmd().arg("+%y").succeeds().stdout_matches(&re);
 }
 
+#[test]
+fn test_date_format_q() {
+    let scene = TestScenario::new(util_name!());
+
+    let re = Regex::new(r"^[1-4]\n$").unwrap();
+    scene.ucmd().arg("+%q").succeeds().stdout_matches(&re);
+}
+
 #[test]
 fn test_date_format_m() {
     let scene = TestScenario::new(util_name!());
@@ -214,9 +225,8 @@ fn test_date_format_without_plus() {
     // [+FORMAT]
     new_ucmd!()
         .arg("%s")
-        .fails()
-        .stderr_contains("date: invalid date '%s'")
-        .code_is(1);
+        .fails_with_code(1)
+        .stderr_contains("date: invalid date '%s'");
 }
 
 #[test]
@@ -267,9 +277,11 @@ fn test_date_set_mac_unavailable() {
         .arg("2020-03-11 21:45:00+08:00")
         .fails();
     result.no_stdout();
-    assert!(result
-        .stderr_str()
-        .starts_with("date: setting the date is not supported by macOS"));
+    assert!(
+        result
+            .stderr_str()
+            .starts_with("date: setting the date is not supported by macOS")
+    );
 }
 
 #[test]
@@ -374,10 +386,12 @@ fn test_invalid_format_string() {
 }
 
 #[test]
-fn test_unsupported_format() {
-    let result = new_ucmd!().arg("+%#z").fails();
-    result.no_stdout();
-    assert!(result.stderr_str().starts_with("date: invalid format %#z"));
+fn test_capitalized_numeric_time_zone() {
+    // %z     +hhmm numeric time zone (e.g., -0400)
+    // # is supposed to capitalize, which makes little sense here, but chrono crashes
+    // on such format so it's good to test.
+    let re = Regex::new(r"^[+-]\d{4,4}\n$").unwrap();
+    new_ucmd!().arg("+%#z").succeeds().stdout_matches(&re);
 }
 
 #[test]
@@ -392,10 +406,13 @@ fn test_date_string_human() {
         "30 minutes ago",
         "10 seconds",
         "last day",
+        "last monday",
         "last week",
         "last month",
         "last year",
+        "this monday",
         "next day",
+        "next monday",
         "next week",
         "next month",
         "next year",
@@ -411,6 +428,61 @@ fn test_date_string_human() {
     }
 }
 
+#[test]
+fn test_negative_offset() {
+    let data_formats = vec![
+        ("-1 hour", Duration::hours(1)),
+        ("-1 hours", Duration::hours(1)),
+        ("-1 day", Duration::days(1)),
+        ("-2 weeks", Duration::weeks(2)),
+    ];
+    for (date_format, offset) in data_formats {
+        new_ucmd!()
+            .arg("-d")
+            .arg(date_format)
+            .arg("--rfc-3339=seconds")
+            .succeeds()
+            .stdout_str_check(|out| {
+                let date = DateTime::parse_from_rfc3339(out.trim()).unwrap();
+
+                // Is the resulting date roughly what is expected?
+                let expected_date = Utc::now() - offset;
+                (date.to_utc() - expected_date).abs() < Duration::minutes(10)
+            });
+    }
+}
+
+#[test]
+fn test_relative_weekdays() {
+    // Truncate time component to midnight
+    let today = Utc::now().with_time(NaiveTime::MIN).unwrap();
+    // Loop through each day of the week, starting with today
+    for offset in 0..7 {
+        for direction in ["last", "this", "next"] {
+            let weekday = (today + Duration::days(offset))
+                .weekday()
+                .to_string()
+                .to_lowercase();
+            new_ucmd!()
+                .arg("-d")
+                .arg(format!("{} {}", direction, weekday))
+                .arg("--rfc-3339=seconds")
+                .arg("--utc")
+                .succeeds()
+                .stdout_str_check(|out| {
+                    let result = DateTime::parse_from_rfc3339(out.trim()).unwrap().to_utc();
+                    let expected = match (direction, offset) {
+                        ("last", _) => today - Duration::days(7 - offset),
+                        ("this", 0) => today,
+                        ("next", 0) => today + Duration::days(7),
+                        _ => today + Duration::days(offset),
+                    };
+                    result == expected
+                });
+        }
+    }
+}
+
 #[test]
 fn test_invalid_date_string() {
     new_ucmd!()
@@ -419,6 +491,15 @@ fn test_invalid_date_string() {
         .fails()
         .no_stdout()
         .stderr_contains("invalid date");
+
+    new_ucmd!()
+        .arg("-d")
+        // cSpell:disable
+        .arg("this fooday")
+        // cSpell:enable
+        .fails()
+        .no_stdout()
+        .stderr_contains("invalid date");
 }
 
 #[test]
