@@ -12,11 +12,15 @@
 
 use std::collections::HashSet;
 
-use crate::common::util::TestScenario;
+#[cfg(not(any(target_os = "freebsd", target_os = "windows")))]
+use uutests::at_and_ucmd;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -285,6 +289,7 @@ fn test_type_option() {
 
 #[test]
 #[cfg(not(any(target_os = "freebsd", target_os = "windows")))] // FIXME: fix test for FreeBSD & Win
+#[cfg(not(feature = "feat_selinux"))]
 fn test_type_option_with_file() {
     let fs_type = new_ucmd!()
         .args(&["--output=fstype", "."])
@@ -298,6 +303,12 @@ fn test_type_option_with_file() {
         .fails()
         .stderr_contains("no file systems processed");
 
+    // Assume the mount point at /dev has a different filesystem type to the mount point at /
+    new_ucmd!()
+        .args(&["-t", fs_type, "/dev"])
+        .fails()
+        .stderr_contains("no file systems processed");
+
     let fs_types = new_ucmd!()
         .arg("--output=fstype")
         .succeeds()
@@ -357,6 +368,10 @@ fn test_include_exclude_same_type() {
         );
 }
 
+#[cfg_attr(
+    all(target_arch = "aarch64", target_os = "linux"),
+    ignore = "Issue #7158 - Test not supported on ARM64 Linux"
+)]
 #[test]
 fn test_total() {
     // Example output:
