@@ -10,25 +10,28 @@
     clippy::cast_sign_loss
 )]
 
-use crate::common::util::TestScenario;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
 
+use std::fmt::Write;
 use std::time::{Duration, SystemTime};
 
-use rand::distributions::{Distribution, Uniform};
-use rand::{rngs::SmallRng, Rng, SeedableRng};
+use rand::distr::{Distribution, Uniform};
+use rand::{Rng, SeedableRng, rngs::SmallRng};
 
 const NUM_PRIMES: usize = 10000;
 const NUM_TESTS: usize = 100;
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
 fn test_valid_arg_exponents() {
-    new_ucmd!().arg("-h").succeeds().code_is(0);
-    new_ucmd!().arg("--exponents").succeeds().code_is(0);
+    new_ucmd!().arg("-h").succeeds();
+    new_ucmd!().arg("--exponents").succeeds();
 }
 
 #[test]
@@ -44,16 +47,16 @@ fn test_repeated_exponents() {
 #[cfg(feature = "sort")]
 #[cfg(not(target_os = "android"))]
 fn test_parallel() {
-    use crate::common::util::AtPath;
     use hex_literal::hex;
     use sha1::{Digest, Sha1};
     use std::{fs::OpenOptions, time::Duration};
     use tempfile::TempDir;
+    use uutests::util::AtPath;
     // factor should only flush the buffer at line breaks
     let n_integers = 100_000;
     let mut input_string = String::new();
     for i in 0..=n_integers {
-        input_string.push_str(&(format!("{i} "))[..]);
+        let _ = write!(input_string, "{i} ");
     }
 
     let tmp_dir = TempDir::new().unwrap();
@@ -98,7 +101,7 @@ fn test_first_1000_integers() {
     let n_integers = 1000;
     let mut input_string = String::new();
     for i in 0..=n_integers {
-        input_string.push_str(&(format!("{i} "))[..]);
+        let _ = write!(input_string, "{i} ");
     }
 
     println!("STDIN='{input_string}'");
@@ -122,7 +125,7 @@ fn test_first_1000_integers_with_exponents() {
     let n_integers = 1000;
     let mut input_string = String::new();
     for i in 0..=n_integers {
-        input_string.push_str(&(format!("{i} "))[..]);
+        let _ = write!(input_string, "{i} ");
     }
 
     println!("STDIN='{input_string}'");
@@ -171,7 +174,7 @@ fn test_random() {
         while product < min {
             // log distribution---higher probability for lower numbers
             let factor = loop {
-                let next = rng.gen_range(0_f64..log_num_primes).exp2().floor() as usize;
+                let next = rng.random_range(0_f64..log_num_primes).exp2().floor() as usize;
                 if next < NUM_PRIMES {
                     break primes[next];
                 }
@@ -183,7 +186,7 @@ fn test_random() {
                     factors.push(factor);
                 }
                 None => break,
-            };
+            }
         }
 
         factors.sort_unstable();
@@ -195,11 +198,11 @@ fn test_random() {
     let mut output_string = String::new();
     for _ in 0..NUM_TESTS {
         let (product, factors) = rand_gt(1 << 63);
-        input_string.push_str(&(format!("{product} "))[..]);
+        let _ = write!(input_string, "{product} ");
 
-        output_string.push_str(&(format!("{product}:"))[..]);
+        let _ = write!(output_string, "{product}:");
         for factor in factors {
-            output_string.push_str(&(format!(" {factor}"))[..]);
+            let _ = write!(output_string, " {factor}");
         }
         output_string.push('\n');
     }
@@ -216,7 +219,7 @@ fn test_random_big() {
     println!("rng_seed={rng_seed:?}");
     let mut rng = SmallRng::seed_from_u64(rng_seed);
 
-    let bit_range_1 = Uniform::new(14_usize, 51);
+    let bit_range_1 = Uniform::new(14, 51).unwrap();
     let mut rand_64 = move || {
         // first, choose a random number of bits for the first factor
         let f_bit_1 = bit_range_1.sample(&mut rng);
@@ -226,11 +229,11 @@ fn test_random_big() {
         // we will have a number of additional factors equal to n_facts + 1
         // where n_facts is in [0, floor(rem/14) )  NOTE half-open interval
         // Each prime factor is at least 14 bits, hence floor(rem/14)
-        let n_factors = Uniform::new(0_usize, rem / 14).sample(&mut rng);
+        let n_factors = Uniform::new(0, rem / 14).unwrap().sample(&mut rng);
         // we have to distribute extra_bits among the (n_facts + 1) values
         let extra_bits = rem - (n_factors + 1) * 14;
         // (remember, a Range is a half-open interval)
-        let extra_range = Uniform::new(0_usize, extra_bits + 1);
+        let extra_range = Uniform::new(0, extra_bits + 1).unwrap();
 
         // to generate an even split of this range, generate n-1 random elements
         // in the range, add the desired total value to the end, sort this list,
@@ -262,7 +265,9 @@ fn test_random_big() {
         for bit in f_bits {
             assert!(bit < 37);
             n_bits += 14 + bit;
-            let elm = Uniform::new(0, PRIMES_BY_BITS[bit].len()).sample(&mut rng);
+            let elm = Uniform::new(0, PRIMES_BY_BITS[bit].len())
+                .unwrap()
+                .sample(&mut rng);
             let factor = PRIMES_BY_BITS[bit][elm];
             factors.push(factor);
             product *= factor;
@@ -277,11 +282,11 @@ fn test_random_big() {
     let mut output_string = String::new();
     for _ in 0..NUM_TESTS {
         let (product, factors) = rand_64();
-        input_string.push_str(&(format!("{product} "))[..]);
+        let _ = write!(input_string, "{product} ");
 
-        output_string.push_str(&(format!("{product}:"))[..]);
+        let _ = write!(output_string, "{product}:");
         for factor in factors {
-            output_string.push_str(&(format!(" {factor}"))[..]);
+            let _ = write!(output_string, " {factor}");
         }
         output_string.push('\n');
     }
@@ -294,8 +299,8 @@ fn test_big_primes() {
     let mut input_string = String::new();
     let mut output_string = String::new();
     for prime in PRIMES64 {
-        input_string.push_str(&(format!("{prime} "))[..]);
-        output_string.push_str(&(format!("{prime}: {prime}\n"))[..]);
+        let _ = write!(input_string, "{prime} ");
+        let _ = writeln!(output_string, "{prime}: {prime}");
     }
 
     run(input_string.as_bytes(), output_string.as_bytes());
@@ -311,7 +316,7 @@ fn run(input_string: &[u8], output_string: &[u8]) {
     new_ucmd!()
         .timeout(Duration::from_secs(240))
         .pipe_in(input_string)
-        .run()
+        .succeeds()
         .stdout_is(String::from_utf8(output_string.to_owned()).unwrap());
 }
 
@@ -321,8 +326,8 @@ fn test_primes_with_exponents() {
     let mut output_string = String::new();
     for primes in PRIMES_BY_BITS {
         for &prime in *primes {
-            input_string.push_str(&(format!("{prime} "))[..]);
-            output_string.push_str(&(format!("{prime}: {prime}\n"))[..]);
+            let _ = write!(input_string, "{prime} ");
+            let _ = writeln!(output_string, "{prime}: {prime}");
         }
     }
 
@@ -340,7 +345,7 @@ fn test_primes_with_exponents() {
         .timeout(Duration::from_secs(240))
         .arg("--exponents")
         .pipe_in(input_string)
-        .run()
+        .succeeds()
         .stdout_is(String::from_utf8(output_string.as_bytes().to_owned()).unwrap());
 }
 
