@@ -4,14 +4,18 @@
 // file that was distributed with this source code.
 //
 // spell-checker:ignore mydir
-use crate::common::util::TestScenario;
 use filetime::FileTime;
 use rstest::rstest;
 use std::io::Write;
+#[cfg(not(windows))]
+use std::path::Path;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::{at_and_ucmd, util_name};
 
 #[test]
 fn test_mv_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -49,6 +53,22 @@ fn test_mv_rename_file() {
     assert!(at.file_exists(file2));
 }
 
+#[test]
+fn test_mv_with_source_file_opened_and_target_file_exists() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let src = "source_file_opened";
+    let dst = "target_file_exists";
+
+    let f = at.make_file(src);
+
+    at.touch(dst);
+
+    ucmd.arg(src).arg(dst).succeeds().no_stderr().no_stdout();
+
+    drop(f);
+}
+
 #[test]
 fn test_mv_move_file_into_dir() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -403,7 +423,7 @@ fn test_mv_same_file() {
     ucmd.arg(file_a)
         .arg(file_a)
         .fails()
-        .stderr_is(format!("mv: '{file_a}' and '{file_a}' are the same file\n",));
+        .stderr_is(format!("mv: '{file_a}' and '{file_a}' are the same file\n"));
 }
 
 #[test]
@@ -420,7 +440,7 @@ fn test_mv_same_hardlink() {
     ucmd.arg(file_a)
         .arg(file_b)
         .fails()
-        .stderr_is(format!("mv: '{file_a}' and '{file_b}' are the same file\n",));
+        .stderr_is(format!("mv: '{file_a}' and '{file_b}' are the same file\n"));
 }
 
 #[test]
@@ -438,7 +458,7 @@ fn test_mv_same_symlink() {
     ucmd.arg(file_b)
         .arg(file_a)
         .fails()
-        .stderr_is(format!("mv: '{file_b}' and '{file_a}' are the same file\n",));
+        .stderr_is(format!("mv: '{file_b}' and '{file_a}' are the same file\n"));
 
     let (at2, mut ucmd2) = at_and_ucmd!();
     at2.touch(file_a);
@@ -938,7 +958,12 @@ fn test_mv_update_option() {
     filetime::set_file_times(at.plus_as_string(file_a), now, now).unwrap();
     filetime::set_file_times(at.plus_as_string(file_b), now, later).unwrap();
 
-    scene.ucmd().arg("--update").arg(file_a).arg(file_b).run();
+    scene
+        .ucmd()
+        .arg("--update")
+        .arg(file_a)
+        .arg(file_b)
+        .succeeds();
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
@@ -1119,6 +1144,30 @@ fn test_mv_arg_update_older_dest_older() {
     assert_eq!(at.read(old), new_content);
 }
 
+#[test]
+fn test_mv_arg_update_older_dest_older_interactive() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let old = "old";
+    let new = "new";
+    let old_content = "file1 content\n";
+    let new_content = "file2 content\n";
+
+    let mut f = at.make_file(old);
+    f.write_all(old_content.as_bytes()).unwrap();
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
+
+    at.write(new, new_content);
+
+    ucmd.arg(new)
+        .arg(old)
+        .arg("--interactive")
+        .arg("--update=older")
+        .fails()
+        .stderr_contains("overwrite 'old'?")
+        .no_stdout();
+}
+
 #[test]
 fn test_mv_arg_update_short_overwrite() {
     // same as --update=older
@@ -1332,13 +1381,15 @@ fn test_mv_errors() {
     // $ at.mkdir dir && at.touch file
     // $ mv dir file
     // err == mv: cannot overwrite non-directory 'file' with directory 'dir'
-    assert!(!scene
-        .ucmd()
-        .arg(dir)
-        .arg(file_a)
-        .fails()
-        .stderr_str()
-        .is_empty());
+    assert!(
+        !scene
+            .ucmd()
+            .arg(dir)
+            .arg(file_a)
+            .fails()
+            .stderr_str()
+            .is_empty()
+    );
 }
 
 #[test]
@@ -1403,15 +1454,17 @@ fn test_mv_interactive_error() {
     // $ at.mkdir dir && at.touch file
     // $ mv -i dir file
     // err == mv: cannot overwrite non-directory 'file' with directory 'dir'
-    assert!(!scene
-        .ucmd()
-        .arg("-i")
-        .arg(dir)
-        .arg(file_a)
-        .pipe_in("y")
-        .fails()
-        .stderr_str()
-        .is_empty());
+    assert!(
+        !scene
+            .ucmd()
+            .arg("-i")
+            .arg(dir)
+            .arg(file_a)
+            .pipe_in("y")
+            .fails()
+            .stderr_str()
+            .is_empty()
+    );
 }
 
 #[test]
@@ -1452,11 +1505,14 @@ fn test_mv_into_self_data() {
     at.touch(file1);
     at.touch(file2);
 
-    let result = scene.ucmd().arg(file1).arg(sub_dir).arg(sub_dir).run();
+    scene
+        .ucmd()
+        .arg(file1)
+        .arg(sub_dir)
+        .arg(sub_dir)
+        .fails_with_code(1);
 
     // sub_dir exists, file1 has been moved, file2 still exists.
-    result.code_is(1);
-
     assert!(at.dir_exists(sub_dir));
     assert!(at.file_exists(file1_result_location));
     assert!(at.file_exists(file2));
@@ -1533,13 +1589,17 @@ fn test_mv_seen_file() {
     let result = ts.ucmd().arg("a/f").arg("b/f").arg("c").fails();
 
     #[cfg(not(unix))]
-    assert!(result
-        .stderr_str()
-        .contains("will not overwrite just-created 'c\\f' with 'b/f'"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("will not overwrite just-created 'c\\f' with 'b/f'")
+    );
     #[cfg(unix)]
-    assert!(result
-        .stderr_str()
-        .contains("will not overwrite just-created 'c/f' with 'b/f'"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("will not overwrite just-created 'c/f' with 'b/f'")
+    );
 
     // a/f has been moved into c/f
     assert!(at.plus("c").join("f").exists());
@@ -1563,13 +1623,17 @@ fn test_mv_seen_multiple_files_to_directory() {
 
     let result = ts.ucmd().arg("a/f").arg("b/f").arg("b/g").arg("c").fails();
     #[cfg(not(unix))]
-    assert!(result
-        .stderr_str()
-        .contains("will not overwrite just-created 'c\\f' with 'b/f'"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("will not overwrite just-created 'c\\f' with 'b/f'")
+    );
     #[cfg(unix)]
-    assert!(result
-        .stderr_str()
-        .contains("will not overwrite just-created 'c/f' with 'b/f'"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("will not overwrite just-created 'c/f' with 'b/f'")
+    );
 
     assert!(!at.plus("a").join("f").exists());
     assert!(at.plus("b").join("f").exists());
@@ -1610,7 +1674,7 @@ fn test_mv_dir_into_path_slash() {
 fn test_acl() {
     use std::process::Command;
 
-    use crate::common::util::compare_xattrs;
+    use uutests::util::compare_xattrs;
 
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -1646,6 +1710,51 @@ fn test_acl() {
     assert!(compare_xattrs(&file, &file_target));
 }
 
+#[test]
+#[cfg(windows)]
+fn test_move_should_not_fallback_to_copy() {
+    use std::os::windows::fs::OpenOptionsExt;
+
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let locked_file = "a_file_is_locked";
+    let locked_file_path = at.plus(locked_file);
+    let file = std::fs::OpenOptions::new()
+        .create(true)
+        .write(true)
+        .share_mode(
+            uucore::windows_sys::Win32::Storage::FileSystem::FILE_SHARE_READ
+                | uucore::windows_sys::Win32::Storage::FileSystem::FILE_SHARE_WRITE,
+        )
+        .open(locked_file_path);
+
+    let target_file = "target_file";
+    ucmd.arg(locked_file).arg(target_file).fails();
+
+    assert!(at.file_exists(locked_file));
+    assert!(!at.file_exists(target_file));
+
+    drop(file);
+}
+
+#[test]
+#[cfg(unix)]
+fn test_move_should_not_fallback_to_copy() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let readonly_dir = "readonly_dir";
+    let locked_file = "readonly_dir/a_file_is_locked";
+    at.mkdir(readonly_dir);
+    at.touch(locked_file);
+    at.set_mode(readonly_dir, 0o555);
+
+    let target_file = "target_file";
+    ucmd.arg(locked_file).arg(target_file).fails();
+
+    assert!(at.file_exists(locked_file));
+    assert!(!at.file_exists(target_file));
+}
+
 // Todo:
 
 // $ at.touch a b
@@ -1661,10 +1770,11 @@ fn test_acl() {
 
 #[cfg(target_os = "linux")]
 mod inter_partition_copying {
-    use crate::common::util::TestScenario;
     use std::fs::{read_to_string, set_permissions, write};
-    use std::os::unix::fs::{symlink, PermissionsExt};
+    use std::os::unix::fs::{PermissionsExt, symlink};
     use tempfile::TempDir;
+    use uutests::util::TestScenario;
+    use uutests::util_name;
 
     // Ensure that the copying code used in an inter-partition move unlinks the destination symlink.
     #[test]
@@ -1703,7 +1813,7 @@ mod inter_partition_copying {
 
         // make sure that file contents in other_fs_file didn't change.
         assert_eq!(
-            read_to_string(&other_fs_file_path,).expect("Unable to read other_fs_file"),
+            read_to_string(&other_fs_file_path).expect("Unable to read other_fs_file"),
             "other fs file contents"
         );
 
@@ -1718,6 +1828,7 @@ mod inter_partition_copying {
     // that it would output the proper error message.
     #[test]
     pub(crate) fn test_mv_unlinks_dest_symlink_error_message() {
+        use uutests::util::TestScenario;
         let scene = TestScenario::new(util_name!());
         let at = &scene.fixtures;
 
@@ -1767,3 +1878,18 @@ fn test_mv_error_msg_with_multiple_sources_that_does_not_exist() {
         .stderr_contains("mv: cannot stat 'a': No such file or directory")
         .stderr_contains("mv: cannot stat 'b/': No such file or directory");
 }
+
+#[cfg(not(windows))]
+#[ignore = "requires access to a different filesystem"]
+#[test]
+fn test_special_file_different_filesystem() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.mkfifo("f");
+    // TODO Use `TestScenario::mount_temp_fs()` for this purpose and
+    // un-ignore this test.
+    std::fs::create_dir("/dev/shm/tmp").unwrap();
+    ucmd.args(&["f", "/dev/shm/tmp"]).succeeds().no_output();
+    assert!(!at.file_exists("f"));
+    assert!(Path::new("/dev/shm/tmp/f").exists());
+    std::fs::remove_dir_all("/dev/shm/tmp").unwrap();
+}
