@@ -5,21 +5,30 @@
 
 // spell-checker:ignore wipesync
 
-use crate::common::util::TestScenario;
+use uutests::at_and_ucmd;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
+
+const PATTERNS: [&str; 22] = [
+    "000000", "ffffff", "555555", "aaaaaa", "249249", "492492", "6db6db", "924924", "b6db6d",
+    "db6db6", "111111", "222222", "333333", "444444", "666666", "777777", "888888", "999999",
+    "bbbbbb", "cccccc", "dddddd", "eeeeee",
+];
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
 fn test_invalid_remove_arg() {
-    new_ucmd!().arg("--remove=unknown").fails().code_is(1);
+    new_ucmd!().arg("--remove=unknown").fails_with_code(1);
 }
 
 #[test]
 fn test_ambiguous_remove_arg() {
-    new_ucmd!().arg("--remove=wip").fails().code_is(1);
+    new_ucmd!().arg("--remove=wip").fails_with_code(1);
 }
 
 #[test]
@@ -36,7 +45,7 @@ fn test_shred() {
     // File exists
     assert!(at.file_exists(file));
     // File is obfuscated
-    assert!(at.read_bytes(file) != file_original_content.as_bytes());
+    assert_ne!(at.read_bytes(file), file_original_content.as_bytes());
 }
 
 #[test]
@@ -126,13 +135,13 @@ fn test_shred_force() {
     at.set_readonly(file);
 
     // Try shred -u.
-    scene.ucmd().arg("-u").arg(file).run();
+    scene.ucmd().arg("-u").arg(file).fails();
 
     // file_a was not deleted because it is readonly.
     assert!(at.file_exists(file));
 
     // Try shred -u -f.
-    scene.ucmd().arg("-u").arg("-f").arg(file).run();
+    scene.ucmd().arg("-u").arg("-f").arg(file).succeeds();
 
     // file_a was deleted.
     assert!(!at.file_exists(file));
@@ -205,3 +214,40 @@ fn test_shred_fail_no_perm() {
         .fails()
         .stderr_contains("Couldn't rename to");
 }
+
+#[test]
+fn test_shred_verbose_no_padding_1() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let file = "foo";
+    at.write(file, "non-empty");
+    ucmd.arg("-vn1")
+        .arg(file)
+        .succeeds()
+        .stderr_only("shred: foo: pass 1/1 (random)...\n");
+}
+
+#[test]
+fn test_shred_verbose_no_padding_10() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let file = "foo";
+    at.write(file, "non-empty");
+    ucmd.arg("-vn10")
+        .arg(file)
+        .succeeds()
+        .stderr_contains("shred: foo: pass 1/10 (random)...\n");
+}
+
+#[test]
+fn test_all_patterns_present() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let file = "foo.txt";
+    at.write(file, "bar");
+
+    let result = scene.ucmd().arg("-vn25").arg(file).succeeds();
+
+    for pat in PATTERNS {
+        result.stderr_contains(pat);
+    }
+}
